@@ -18,8 +18,8 @@ export SOURCE_LEVEL=1.6
 export TARGET_LEVEL=1.6
 export TARGET_RT_JAR=/opt-share/jre1.6.0_30/lib/rt.jar
 
-export GLUEGEN_PROPERTIES_FILE="lib/gluegen-clang.properties"
-# or -Dgcc.compat.compiler=clang
+export GLUEGEN_PROPERTIES_FILE="lib/gluegen-xcode_clang.properties"
+# or -Dgcc.compat.compiler=xcode.clang
 
 #export JOGAMP_JAR_CODEBASE="Codebase: *.jogamp.org"
 export JOGAMP_JAR_CODEBASE="Codebase: *.goethel.localnet"
