@@ -47,22 +47,25 @@ import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 
 import com.jogamp.common.os.NativeLibrary;
+import com.jogamp.common.os.Platform;
 
 import jogamp.common.Debug;
 
 public class JarUtil {
     private static final boolean DEBUG = Debug.debug("JarUtil");
 
+    private static final int BUFFER_SIZE = 4096;
+
     /**
      * Interface allowing users to provide an URL resolver that will convert custom classloader
      * URLs like Eclipse/OSGi <i>bundleresource:</i> URLs to normal <i>jar:</i> URLs.
-     * <p> 
+     * <p>
      * This might be required for custom classloader where the URI protocol is unknown
-     * to the standard runtime environment. 
+     * to the standard runtime environment.
      * </p>
      * <p>
      * Note: The provided resolver is only utilized if a given URI's protocol could not be resolved.
-     * I.e. it will not be invoked for known protocols like <i>http</i>, <i>https</i>, <i>jar</i> or <i>file</i>. 
+     * I.e. it will not be invoked for known protocols like <i>http</i>, <i>https</i>, <i>jar</i> or <i>file</i>.
      * </p>
      */
     public interface Resolver {
@@ -70,15 +73,15 @@ public class JarUtil {
     }
 
     private static Resolver resolver;
-    
+
     /**
      * Setting a custom {@link Resolver} instance.
-     * 
+     *
      * @param r {@link Resolver} to use after querying class file URLs from the classloader.
      * @throws IllegalArgumentException if the passed resolver is <code>null</code>
      * @throws IllegalStateException if the resolver has already been set.
      * @throws SecurityException if the security manager doesn't have the setFactory
-     * permission 
+     * permission
      */
     public static void setResolver(Resolver r) throws IllegalArgumentException, IllegalStateException, SecurityException {
         if(r == null) {
@@ -99,13 +102,13 @@ public class JarUtil {
 
     /**
      * Returns <code>true</code> if the Class's <code>"com.jogamp.common.GlueGenVersion"</code>
-     * is loaded from a JarFile and hence has a Jar URI like 
+     * is loaded from a JarFile and hence has a Jar URI like
      * URI <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class"</code>.
      * <p>
      * <i>sub_protocol</i> may be "file", "http", etc..
      * </p>
-     * 
-     * @param clazzBinName "com.jogamp.common.GlueGenVersion" 
+     *
+     * @param clazzBinName "com.jogamp.common.GlueGenVersion"
      * @param cl
      * @return true if the class is loaded from a Jar file, otherwise false.
      * @see {@link #getJarURI(String, ClassLoader)}
@@ -116,20 +119,20 @@ public class JarUtil {
         } catch (Exception e) { /* ignore */ }
         return false;
     }
-    
+
     /**
-     * The Class's <code>"com.jogamp.common.GlueGenVersion"</code> 
+     * The Class's <code>"com.jogamp.common.GlueGenVersion"</code>
      * URI <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class"</code>
      * will be returned.
      * <p>
      * <i>sub_protocol</i> may be "file", "http", etc..
      * </p>
-     * 
-     * @param clazzBinName "com.jogamp.common.GlueGenVersion" 
+     *
+     * @param clazzBinName "com.jogamp.common.GlueGenVersion"
      * @param cl ClassLoader to locate the JarFile
      * @return "jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class"
      * @throws IllegalArgumentException if the URI doesn't match the expected formatting or null arguments
-     * @throws IOException if the class's Jar file could not been found by the ClassLoader 
+     * @throws IOException if the class's Jar file could not been found by the ClassLoader
      * @throws URISyntaxException if the URI could not be translated into a RFC 2396 URI
      * @see {@link IOUtil#getClassURL(String, ClassLoader)}
      */
@@ -138,24 +141,25 @@ public class JarUtil {
             throw new IllegalArgumentException("null arguments: clazzBinName "+clazzBinName+", cl "+cl);
         }
         final URI uri;
+        final URL url;
         {
-            final URL url = IOUtil.getClassURL(clazzBinName, cl);
+            url = IOUtil.getClassURL(clazzBinName, cl);
             final String scheme = url.getProtocol();
             if( null != resolver &&
                 !scheme.equals( IOUtil.JAR_SCHEME ) &&
                 !scheme.equals( IOUtil.FILE_SCHEME ) &&
                 !scheme.equals( IOUtil.HTTP_SCHEME ) &&
-                !scheme.equals( IOUtil.HTTPS_SCHEME ) ) 
+                !scheme.equals( IOUtil.HTTPS_SCHEME ) )
             {
                 final URL _url = resolver.resolve( url );
                 uri = _url.toURI();
                 if(DEBUG) {
-                    System.out.println("getJarURI Resolver: "+url+"\n\t-> "+_url+"\n\t-> "+uri);
+                    System.err.println("getJarURI Resolver: "+url+"\n\t-> "+_url+"\n\t-> "+uri);
                 }
             } else {
                 uri = url.toURI();
                 if(DEBUG) {
-                    System.out.println("getJarURI Default "+url+"\n\t-> "+uri);
+                    System.err.println("getJarURI Default "+url+"\n\t-> "+uri);
                 }
             }
         }
@@ -163,21 +167,24 @@ public class JarUtil {
         if( !uri.getScheme().equals( IOUtil.JAR_SCHEME ) ) {
             throw new IllegalArgumentException("URI is not using scheme "+IOUtil.JAR_SCHEME+": <"+uri+">");
         }
+        if(DEBUG) {
+            System.err.println("getJarURI res: "+clazzBinName+" -> "+url+" -> "+uri);
+        }
         return uri;
     }
-    
-    
+
+
     /**
      * The Class's Jar URI <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class</code>
      * Jar basename <code>gluegen-rt.jar</code> will be returned.
      * <p>
      * <i>sub_protocol</i> may be "file", "http", etc..
      * </p>
-     * 
-     * @param classJarURI as retrieved w/ {@link #getJarURI(String, ClassLoader) getJarURI("com.jogamp.common.GlueGenVersion", cl).toURI()}, 
+     *
+     * @param classJarURI as retrieved w/ {@link #getJarURI(String, ClassLoader) getJarURI("com.jogamp.common.GlueGenVersion", cl).toURI()},
      *                    i.e. <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class</code>
      * @return <code>gluegen-rt.jar</code>
-     * @throws IllegalArgumentException if the URI doesn't match the expected formatting or is null 
+     * @throws IllegalArgumentException if the URI doesn't match the expected formatting or is null
      * @see {@link IOUtil#getClassURL(String, ClassLoader)}
      */
     public static String getJarBasename(URI classJarURI) throws IllegalArgumentException {
@@ -187,20 +194,20 @@ public class JarUtil {
         if( !classJarURI.getScheme().equals(IOUtil.JAR_SCHEME) ) {
             throw new IllegalArgumentException("URI is not using scheme "+IOUtil.JAR_SCHEME+": <"+classJarURI+">");
         }
-        String uriS = classJarURI.getRawSchemeSpecificPart();
-        
-        // from 
+        String uriS = classJarURI.getSchemeSpecificPart();
+
+        // from
         //   file:/some/path/gluegen-rt.jar!/com/jogamp/common/util/cache/TempJarCache.class
         // to
         //   file:/some/path/gluegen-rt.jar
-        int idx = uriS.lastIndexOf('!');
+        int idx = uriS.lastIndexOf(IOUtil.JAR_SCHEME_SEPARATOR);
         if (0 <= idx) {
-            uriS = uriS.substring(0, idx); // exclude '!/'                
+            uriS = uriS.substring(0, idx); // exclude '!/'
         } else {
-            throw new IllegalArgumentException("JAR URI does not contain jar uri terminator '!', in <"+classJarURI+">");
+            throw new IllegalArgumentException("URI does not contain jar uri terminator '!', in <"+classJarURI+">");
         }
-        
-        // from 
+
+        // from
         //   file:/some/path/gluegen-rt.jar
         // to
         //   gluegen-rt.jar
@@ -209,32 +216,32 @@ public class JarUtil {
             // no abs-path, check for protocol terminator ':'
             idx = uriS.lastIndexOf(':');
             if(0 > idx) {
-                throw new IllegalArgumentException("JAR URI does not contain protocol terminator ':', in <"+classJarURI+">");
+                throw new IllegalArgumentException("URI does not contain protocol terminator ':', in <"+classJarURI+">");
             }
         }
         uriS = uriS.substring(idx+1); // just the jar name
-        
+
         if(0 >= uriS.lastIndexOf(".jar")) {
             throw new IllegalArgumentException("No Jar name in <"+classJarURI+">");
-        }                    
+        }
         if(DEBUG) {
-            System.out.println("getJarName res: "+uriS);
+            System.err.println("getJarName res: "+uriS);
         }
         return uriS;
     }
 
     /**
-     * The Class's <code>com.jogamp.common.GlueGenVersion</code> 
+     * The Class's <code>com.jogamp.common.GlueGenVersion</code>
      * URI <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class</code>
      * Jar basename <code>gluegen-rt.jar</code> will be returned.
      * <p>
      * <i>sub_protocol</i> may be "file", "http", etc..
      * </p>
-     * 
+     *
      * @param clazzBinName <code>com.jogamp.common.GlueGenVersion</code>
      * @param cl
      * @return <code>gluegen-rt.jar</code>
-     * @throws IllegalArgumentException if the URI doesn't match the expected formatting 
+     * @throws IllegalArgumentException if the URI doesn't match the expected formatting
      * @throws IOException if the class's Jar file could not been found by the ClassLoader.
      * @throws URISyntaxException if the URI could not be translated into a RFC 2396 URI
      * @see {@link IOUtil#getClassURL(String, ClassLoader)}
@@ -242,93 +249,99 @@ public class JarUtil {
     public static String getJarBasename(String clazzBinName, ClassLoader cl) throws IllegalArgumentException, IOException, URISyntaxException {
         return getJarBasename( getJarURI(clazzBinName, cl) );
     }
-    
+
     /**
      * The Class's Jar URI <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class</code>
      * Jar file's sub URI <code><i>sub_protocol</i>:/some/path/gluegen-rt.jar</code> will be returned.
      * <p>
      * <i>sub_protocol</i> may be "file", "http", etc..
      * </p>
-     * 
-     * @param classJarURI as retrieved w/ {@link #getJarURI(String, ClassLoader) getJarURI("com.jogamp.common.GlueGenVersion", cl).toURI()}, 
+     *
+     * @param classJarURI as retrieved w/ {@link #getJarURI(String, ClassLoader) getJarURI("com.jogamp.common.GlueGenVersion", cl).toURI()},
      *                    i.e. <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class</code>
      * @return <code><i>sub_protocol</i>:/some/path/gluegen-rt.jar</code>
-     * @throws IllegalArgumentException if the URI doesn't match the expected formatting or is null 
+     * @throws IllegalArgumentException if the URI doesn't match the expected formatting or is null
      * @throws URISyntaxException if the URI could not be translated into a RFC 2396 URI
      * @see {@link IOUtil#getClassURL(String, ClassLoader)}
      */
     public static URI getJarSubURI(URI classJarURI) throws IllegalArgumentException, URISyntaxException {
         if(null == classJarURI) {
-            throw new IllegalArgumentException("URI is null");            
+            throw new IllegalArgumentException("URI is null");
         }
         if( !classJarURI.getScheme().equals(IOUtil.JAR_SCHEME) ) {
             throw new IllegalArgumentException("URI is not a using scheme "+IOUtil.JAR_SCHEME+": <"+classJarURI+">");
         }
-        String uriS = classJarURI.getRawSchemeSpecificPart();
-        
-        // from 
+
+        // from
         //   file:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class
         // to
         //   file:/some/path/gluegen-rt.jar
-        int idx = uriS.lastIndexOf('!');
+        final String uriS0 = classJarURI.getSchemeSpecificPart();
+        int idx = uriS0.lastIndexOf(IOUtil.JAR_SCHEME_SEPARATOR);
+        final String uriS1;
         if (0 <= idx) {
-            uriS = uriS.substring(0, idx); // exclude '!/'
+            uriS1 = uriS0.substring(0, idx); // exclude '!/'
         } else {
             throw new IllegalArgumentException("JAR URI does not contain jar uri terminator '!', uri <"+classJarURI+">");
         }
-        
-        if(0 >= uriS.lastIndexOf(".jar")) {
+        if(0 >= uriS1.lastIndexOf(".jar")) {
             throw new IllegalArgumentException("No Jar name in <"+classJarURI+">");
-        }                    
-        if(DEBUG) {
-            System.out.println("getJarSubURI res: "+uriS);
         }
-        return new URI(uriS);
+        final String uriS2 = IOUtil.encodeToURI(uriS1); // 'new URI(String)' will not encode space!
+        if(DEBUG) {
+            System.err.println("getJarSubURI res: "+classJarURI+" -> "+uriS0+" -> "+uriS1+" -> "+uriS2);
+        }
+        return new URI(uriS2);
     }
-    
+
     /**
      * The Class's Jar URI <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class</code>
      * Jar file's entry <code>/com/jogamp/common/GlueGenVersion.class</code> will be returned.
-     * 
-     * @param classJarURI as retrieved w/ {@link #getJarURI(String, ClassLoader) getJarURI("com.jogamp.common.GlueGenVersion", cl).toURI()}, 
+     *
+     * @param classJarURI as retrieved w/ {@link #getJarURI(String, ClassLoader) getJarURI("com.jogamp.common.GlueGenVersion", cl).toURI()},
      *                    i.e. <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class</code>
      * @return <code>/com/jogamp/common/GlueGenVersion.class</code>
      * @see {@link IOUtil#getClassURL(String, ClassLoader)}
      */
     public static String getJarEntry(URI classJarURI) {
         if(null == classJarURI) {
-            throw new IllegalArgumentException("URI is null");            
+            throw new IllegalArgumentException("URI is null");
         }
         if( !classJarURI.getScheme().equals(IOUtil.JAR_SCHEME) ) {
             throw new IllegalArgumentException("URI is not a using scheme "+IOUtil.JAR_SCHEME+": <"+classJarURI+">");
         }
-        String uriS = classJarURI.getRawSchemeSpecificPart();
-        
-        // from 
+        // final String uriS = classJarURI.toString(); // getSchemeSpecificPart(); ignore fragment !
+        final String uriS = classJarURI.getSchemeSpecificPart();
+
+        // from
         //   file:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class
         // to
-        //   file:/some/path/gluegen-rt.jar
-        int idx = uriS.lastIndexOf('!');
+        //   /com/jogamp/common/GlueGenVersion.class
+        final int idx = uriS.lastIndexOf(IOUtil.JAR_SCHEME_SEPARATOR);
         if (0 <= idx) {
-            return uriS.substring(idx+1); // right of '!'
+            final String res = uriS.substring(idx+1); // right of '!'
+            if(DEBUG) {
+                System.err.println("getJarEntry res: "+classJarURI+" -> "+uriS+" -> "+idx+" -> "+res);
+            }
+            return res;
         } else {
             throw new IllegalArgumentException("JAR URI does not contain jar uri terminator '!', uri <"+classJarURI+">");
         }
     }
 
     /**
-     * The Class's <code>com.jogamp.common.GlueGenVersion</code> 
+     * The Class's <code>com.jogamp.common.GlueGenVersion</code>
      * URI <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class</code>
      * Jar file's sub URI <code><i>sub_protocol</i>:/some/path/gluegen-rt.jar</code> will be returned.
      * <p>
      * <i>sub_protocol</i> may be "file", "http", etc..
      * </p>
-     * 
+     *
      * @param clazzBinName <code>com.jogamp.common.GlueGenVersion</code>
      * @param cl
      * @return <code><i>sub_protocol</i>:/some/path/gluegen-rt.jar</code>
-     * @throws IllegalArgumentException if the URI doesn't match the expected formatting 
-     * @throws IOException if the class's Jar file could not been found by the ClassLoader 
+     * @throws IllegalArgumentException if the URI doesn't match the expected formatting
+     * @throws IOException if the class's Jar file could not been found by the ClassLoader
      * @throws URISyntaxException if the URI could not be translated into a RFC 2396 URI
      * @see {@link IOUtil#getClassURL(String, ClassLoader)}
      */
@@ -337,18 +350,18 @@ public class JarUtil {
     }
 
     /**
-     * The Class's <code>"com.jogamp.common.GlueGenVersion"</code> 
+     * The Class's <code>"com.jogamp.common.GlueGenVersion"</code>
      * URI <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class"</code>
      * Jar file URI <code>jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/</code> will be returned.
      * <p>
      * <i>sub_protocol</i> may be "file", "http", etc..
      * </p>
-     * 
-     * @param clazzBinName "com.jogamp.common.GlueGenVersion" 
+     *
+     * @param clazzBinName "com.jogamp.common.GlueGenVersion"
      * @param cl
      * @return "jar:<i>sub_protocol</i>:/some/path/gluegen-rt.jar!/"
-     * @throws IllegalArgumentException if the URI doesn't match the expected formatting or null arguments 
-     * @throws IOException if the class's Jar file could not been found by the ClassLoader 
+     * @throws IllegalArgumentException if the URI doesn't match the expected formatting or null arguments
+     * @throws IOException if the class's Jar file could not been found by the ClassLoader
      * @throws URISyntaxException if the URI could not be translated into a RFC 2396 URI
      * @see {@link IOUtil#getClassURL(String, ClassLoader)}
      */
@@ -358,7 +371,7 @@ public class JarUtil {
         }
         final URI uri = new URI(IOUtil.JAR_SCHEME, getJarSubURI(clazzBinName, cl).toString()+"!/", null);
         if(DEBUG) {
-            System.out.println("getJarFileURI res: "+uri);
+            System.err.println("getJarFileURI res: "+uri);
         }
         return uri;
     }
@@ -367,7 +380,7 @@ public class JarUtil {
      * @param baseUri file:/some/path/
      * @param jarFileName gluegen-rt.jar
      * @return jar:file:/some/path/gluegen-rt.jar!/
-     * @throws URISyntaxException 
+     * @throws URISyntaxException
      * @throws IllegalArgumentException null arguments
      */
     public static URI getJarFileURI(URI baseUri, String jarFileName) throws IllegalArgumentException, URISyntaxException {
@@ -376,12 +389,12 @@ public class JarUtil {
         }
         return new URI(IOUtil.JAR_SCHEME, baseUri.toString()+jarFileName+"!/", null);
     }
-    
+
     /**
      * @param jarSubUri file:/some/path/gluegen-rt.jar
      * @return jar:file:/some/path/gluegen-rt.jar!/
      * @throws IllegalArgumentException null arguments
-     * @throws URISyntaxException 
+     * @throws URISyntaxException
      */
     public static URI getJarFileURI(URI jarSubUri) throws IllegalArgumentException, URISyntaxException {
         if(null == jarSubUri) {
@@ -389,13 +402,26 @@ public class JarUtil {
         }
         return new URI(IOUtil.JAR_SCHEME, jarSubUri.toString()+"!/", null);
     }
-    
+
+    /**
+     * @param jarSubUriS file:/some/path/gluegen-rt.jar
+     * @return jar:file:/some/path/gluegen-rt.jar!/
+     * @throws IllegalArgumentException null arguments
+     * @throws URISyntaxException
+     */
+    public static URI getJarFileURI(String jarSubUriS) throws IllegalArgumentException, URISyntaxException {
+        if(null == jarSubUriS) {
+            throw new IllegalArgumentException("jarSubURIS is null");
+        }
+        return new URI(IOUtil.JAR_SCHEME, jarSubUriS+"!/", null);
+    }
+
     /**
      * @param jarFileURI jar:file:/some/path/gluegen-rt.jar!/
      * @param jarEntry com/jogamp/common/GlueGenVersion.class
      * @return jar:file:/some/path/gluegen-rt.jar!/com/jogamp/common/GlueGenVersion.class
      * @throws IllegalArgumentException null arguments
-     * @throws URISyntaxException 
+     * @throws URISyntaxException
      */
     public static URI getJarEntryURI(URI jarFileURI, String jarEntry) throws IllegalArgumentException, URISyntaxException {
         if(null == jarEntry) {
@@ -403,12 +429,12 @@ public class JarUtil {
         }
         return new URI(jarFileURI.toString()+jarEntry);
     }
-    
+
     /**
-     * @param clazzBinName com.jogamp.common.util.cache.TempJarCache 
-     * @param cl domain 
+     * @param clazzBinName com.jogamp.common.util.cache.TempJarCache
+     * @param cl domain
      * @return JarFile containing the named class within the given ClassLoader
-     * @throws IOException if the class's Jar file could not been found by the ClassLoader 
+     * @throws IOException if the class's Jar file could not been found by the ClassLoader
      * @throws IllegalArgumentException null arguments
      * @throws URISyntaxException if the URI could not be translated into a RFC 2396 URI
      * @see {@link #getJarFileURI(String, ClassLoader)}
@@ -421,33 +447,102 @@ public class JarUtil {
      * @param jarFileURI jar:file:/some/path/gluegen-rt.jar!/
      * @return JarFile as named by URI within the given ClassLoader
      * @throws IllegalArgumentException null arguments
-     * @throws IOException if the Jar file could not been found 
-     * @throws URISyntaxException 
+     * @throws IOException if the Jar file could not been found
+     * @throws URISyntaxException
      */
     public static JarFile getJarFile(URI jarFileURI) throws IOException, IllegalArgumentException, URISyntaxException {
         if(null == jarFileURI) {
             throw new IllegalArgumentException("null jarFileURI");
         }
         if(DEBUG) {
-            System.out.println("getJarFile: "+jarFileURI.toString());
+            System.err.println("getJarFile.0: "+jarFileURI.toString());
         }
         final URL jarFileURL = IOUtil.toURL(jarFileURI);
-        // final URL jarFileURL = jarFileURI.toURL(); // doesn't work due to encoded path even w/ file schema! 
+        if(DEBUG) {
+            System.err.println("getJarFile.1: "+jarFileURL.toString());
+        }
+        // final URL jarFileURL = jarFileURI.toURL(); // doesn't work due to encoded path even w/ file schema!
         final URLConnection urlc = jarFileURL.openConnection();
         if(urlc instanceof JarURLConnection) {
             JarURLConnection jarConnection = (JarURLConnection)jarFileURL.openConnection();
             JarFile jarFile = jarConnection.getJarFile();
             if(DEBUG) {
-                System.out.println("getJarFile res: "+jarFile.getName());
-            }        
+                System.err.println("getJarFile res: "+jarFile.getName());
+            }
             return jarFile;
         }
         if(DEBUG) {
-            System.out.println("getJarFile res: NULL");
-        }        
+            System.err.println("getJarFile res: NULL");
+        }
         return null;
     }
-    
+
+    /**
+     * Locates the {@link JarUtil#getJarFileURI(URI) Jar file URI} of a given resource
+     * relative to a given class's Jar's URI.
+     * <pre>
+     *   class's jar url path + cutOffInclSubDir + relResPath,
+     * </pre>
+     * Example #1
+     * <pre>
+     *   classFromJavaJar = com.lighting.Test (in: file:/storage/TestLighting.jar)
+     *   cutOffInclSubDir = lights/
+     *   relResPath       = LightAssets.jar
+     *   Result           : file:/storage/lights/LightAssets.jar
+     * </pre>
+     * Example #2
+     * <pre>
+     *   classFromJavaJar = com.lighting.Test (in: file:/storage/lights/TestLighting.jar)
+     *   cutOffInclSubDir = lights/
+     *   relResPath       = LightAssets.jar
+     *   Result           : file:/storage/lights/LightAssets.jar
+     * </pre>
+     *
+     * TODO: Enhance documentation!
+     *
+     * @param classFromJavaJar Used to get the root URI for the class's Jar URI.
+     * @param cutOffInclSubDir The <i>cut off</i> included sub-directory prepending the relative resource path.
+     *                         If the root URI includes cutOffInclSubDir, it is no more added to the result.
+     * @param relResPath The relative resource path.
+     * @return The resulting resource URI, which is not tested.
+     * @throws IllegalArgumentException
+     * @throws IOException
+     * @throws URISyntaxException
+     */
+    public static URI getRelativeOf(Class<?> classFromJavaJar, String cutOffInclSubDir, String relResPath) throws IllegalArgumentException, IOException, URISyntaxException {
+        final ClassLoader cl = classFromJavaJar.getClassLoader();
+        final URI classJarURI = JarUtil.getJarURI(classFromJavaJar.getName(), cl);
+        if( DEBUG ) {
+            System.err.println("JarUtil.getRelativeOf: "+"(classFromJavaJar "+classFromJavaJar+", classJarURI "+classJarURI+
+                    ", cutOffInclSubDir "+cutOffInclSubDir+", relResPath "+relResPath+"): ");
+        }
+
+        final URI jarSubURI = JarUtil.getJarSubURI( classJarURI );
+        if(null == jarSubURI) {
+            throw new IllegalArgumentException("JarSubURI is null of: "+classJarURI);
+        }
+        final String jarUriRoot_s = IOUtil.getURIDirname( jarSubURI.toString() );
+        if( DEBUG ) {
+            System.err.println("JarUtil.getRelativeOf: "+"uri "+jarSubURI.toString()+" -> "+jarUriRoot_s);
+        }
+
+        final String resUri_s;
+        if( jarUriRoot_s.endsWith(cutOffInclSubDir) ) {
+            resUri_s = jarUriRoot_s+relResPath;
+        } else {
+            resUri_s = jarUriRoot_s+cutOffInclSubDir+relResPath;
+        }
+        if( DEBUG ) {
+            System.err.println("JarUtil.getRelativeOf: "+"...  -> "+resUri_s);
+        }
+
+        final URI resURI = JarUtil.getJarFileURI(resUri_s);
+        if( DEBUG ) {
+            System.err.println("JarUtil.getRelativeOf: "+"fin "+resURI);
+        }
+        return resURI;
+    }
+
     /**
      * Return a map from native-lib-base-name to entry-name.
      */
@@ -458,12 +553,12 @@ public class JarUtil {
 
         final Map<String,String> nameMap = new HashMap<String, String>();
         final Enumeration<JarEntry> entries = jarFile.entries();
-        
+
         while (entries.hasMoreElements()) {
             final JarEntry entry = entries.nextElement();
             final String entryName = entry.getName();
             final String baseName =  NativeLibrary.isValidNativeLibraryName(entryName, false);
-            
+
             if(null != baseName) {
                 nameMap.put(baseName, entryName);
             }
@@ -483,7 +578,7 @@ public class JarUtil {
      * Root entries are favored over non root entries in case of naming collisions.<br>
      * Example on a Unix like machine:<br>
      * <pre>
-     *   mylib.jar!/sub1/libsour.so   -> sour  (mapped, unique name)  
+     *   mylib.jar!/sub1/libsour.so   -> sour  (mapped, unique name)
      *   mylib.jar!/sub1/libsweet.so           (dropped, root entry favored)
      *   mylib.jar!/libsweet.so       -> sweet (mapped, root entry favored)
      *   mylib.jar!/sweet.dll         ->       (dropped, not a unix library name)
@@ -495,26 +590,27 @@ public class JarUtil {
      * In this case, set all flags to true <code>extractNativeLibraries </code>.
      * <code>extractClassFiles</code>, <code>extractOtherFiles</code>.
      * </p>
-     * 
+     *
      * @param dest
      * @param nativeLibMap
      * @param jarFile
-     * @param deepDirectoryTraversal
+     * @param nativeLibraryPath if not null, only extracts native libraries within this path.
      * @param extractNativeLibraries
      * @param extractClassFiles
      * @param extractOtherFiles
+     * @param deepDirectoryTraversal
      * @return
      * @throws IOException
      */
-    public static final int extract(File dest, Map<String, String> nativeLibMap, 
+    public static final int extract(File dest, Map<String, String> nativeLibMap,
                                     JarFile jarFile,
+                                    String nativeLibraryPath,
                                     boolean extractNativeLibraries,
-                                    boolean extractClassFiles,
-                                    boolean extractOtherFiles) throws IOException {
+                                    boolean extractClassFiles, boolean extractOtherFiles) throws IOException {
 
         if (DEBUG) {
             System.err.println("JarUtil: extract: "+jarFile.getName()+" -> "+dest+
-                               ", extractNativeLibraries "+extractNativeLibraries+
+                               ", extractNativeLibraries "+extractNativeLibraries+" ("+nativeLibraryPath+")"+
                                ", extractClassFiles "+extractClassFiles+
                                ", extractOtherFiles "+extractOtherFiles);
         }
@@ -528,13 +624,31 @@ public class JarUtil {
             // Match entries with correct prefix and suffix (ignoring case)
             final String libBaseName = NativeLibrary.isValidNativeLibraryName(entryName, false);
             final boolean isNativeLib = null != libBaseName;
-            if(isNativeLib && !extractNativeLibraries) {
-                if (DEBUG) {
-                    System.err.println("JarUtil: JarEntry : " + entryName + " native-lib skipped");
+            if(isNativeLib) {
+                if(!extractNativeLibraries) {
+                    if (DEBUG) {
+                        System.err.println("JarUtil: JarEntry : " + entryName + " native-lib skipped, skip all native libs");
+                    }
+                    continue;
+                }
+                if(null != nativeLibraryPath) {
+                    final String nativeLibraryPathS;
+                    final String dirnameS;
+                    try {
+                        nativeLibraryPathS = IOUtil.slashify(nativeLibraryPath, false /* startWithSlash */, true /* endWithSlash */);
+                        dirnameS = IOUtil.getDirname(entryName);
+                    } catch (URISyntaxException e) {
+                        throw new IOException(e);
+                    }
+                    if( !nativeLibraryPathS.equals(dirnameS) ) {
+                        if (DEBUG) {
+                            System.err.println("JarUtil: JarEntry : " + entryName + " native-lib skipped, not in path: "+nativeLibraryPathS);
+                        }
+                        continue;
+                    }
                 }
-                continue;
             }
-            
+
             final boolean isClassFile = entryName.endsWith(".class");
             if(isClassFile && !extractClassFiles) {
                 if (DEBUG) {
@@ -542,7 +656,7 @@ public class JarUtil {
                 }
                 continue;
             }
-            
+
             if(!isNativeLib && !isClassFile && !extractOtherFiles) {
                 if (DEBUG) {
                     System.err.println("JarUtil: JarEntry : " + entryName + " other-file skipped");
@@ -551,35 +665,35 @@ public class JarUtil {
             }
 
             final boolean isDir = entryName.endsWith("/");
-            
-            final boolean isRootEntry = entryName.indexOf('/') == -1 && 
+
+            final boolean isRootEntry = entryName.indexOf('/') == -1 &&
                                         entryName.indexOf(File.separatorChar) == -1;
-            
+
             if (DEBUG) {
-                System.err.println("JarUtil: JarEntry : isNativeLib " + isNativeLib + 
+                System.err.println("JarUtil: JarEntry : isNativeLib " + isNativeLib +
                                    ", isClassFile " + isClassFile + ", isDir " + isDir +
                                    ", isRootEntry " + isRootEntry );
             }
-            
+
             final File destFile = new File(dest, entryName);
             if(isDir) {
                 if (DEBUG) {
                     System.err.println("JarUtil: MKDIR: " + entryName + " -> " + destFile );
                 }
-                destFile.mkdir();
+                destFile.mkdirs();
             } else {
                 final File destFolder = new File(destFile.getParent());
                 if(!destFolder.exists()) {
                     if (DEBUG) {
                         System.err.println("JarUtil: MKDIR (parent): " + entryName + " -> " + destFolder );
-                    }                    
-                    destFolder.mkdir();
+                    }
+                    destFolder.mkdirs();
                 }
                 final InputStream in = new BufferedInputStream(jarFile.getInputStream(entry));
                 final OutputStream out = new BufferedOutputStream(new FileOutputStream(destFile));
-                int numBytes = -1; 
+                int numBytes = -1;
                 try {
-                    numBytes = IOUtil.copyStream2Stream(in, out, -1);
+                    numBytes = IOUtil.copyStream2Stream(BUFFER_SIZE, in, out, -1);
                 } finally {
                     in.close();
                     out.close();
@@ -587,9 +701,10 @@ public class JarUtil {
                 boolean addedAsNativeLib = false;
                 if (numBytes>0) {
                     num++;
-                    if (isNativeLib && ( isRootEntry || !nativeLibMap.containsKey(libBaseName) ) ) {                    
+                    if (isNativeLib && ( isRootEntry || !nativeLibMap.containsKey(libBaseName) ) ) {
                         nativeLibMap.put(libBaseName, destFile.getAbsolutePath());
                         addedAsNativeLib = true;
+                        fixNativeLibAttribs(destFile);
                     }
                 }
                 if (DEBUG) {
@@ -600,6 +715,32 @@ public class JarUtil {
         return num;
     }
 
+    /**
+     * Mitigate file permission issues of native library files, i.e.:
+     * <ul>
+     *   <li>Bug 865: Safari >= 6.1 [OSX]: May employ xattr on 'com.apple.quarantine' on 'PluginProcess.app'</li>
+     * </ul>
+     */
+    private final static void fixNativeLibAttribs(File file) {
+        // We tolerate UnsatisfiedLinkError (and derived) to solve the chicken and egg problem
+        // of loading gluegen's native library.
+        // On Safari(OSX), Bug 865, we simply hope the destination folder is executable.
+        if( Platform.OSType.MACOS == Platform.getOSType() ) {
+            final String fileAbsPath = file.getAbsolutePath();
+            try {
+                fixNativeLibAttribs(fileAbsPath);
+                if( DEBUG ) {
+                    System.err.println("JarUtil.fixNativeLibAttribs: "+fileAbsPath+" - OK");
+                }
+            } catch (Throwable t) {
+                if( DEBUG ) {
+                    System.err.println("JarUtil.fixNativeLibAttribs: "+fileAbsPath+" - "+t.getClass().getSimpleName()+": "+t.getMessage());
+                }
+            }
+        }
+    }
+    private native static boolean fixNativeLibAttribs(String fname);
+
     /**
      * Validate the certificates for each native Lib in the jar file.
      * Throws an IOException if any certificate is not valid.
@@ -608,7 +749,7 @@ public class JarUtil {
                                         getCodeSource().getCertificates();
        </pre>
      */
-    public static final void validateCertificates(Certificate[] rootCerts, JarFile jarFile) 
+    public static final void validateCertificates(Certificate[] rootCerts, JarFile jarFile)
             throws IOException, SecurityException {
 
         if (DEBUG) {
@@ -634,7 +775,7 @@ public class JarUtil {
      * Check the certificates with the ones in the jar file
      * (all must match).
      */
-    private static final void validateCertificate(Certificate[] rootCerts, 
+    private static final void validateCertificate(Certificate[] rootCerts,
             JarFile jar, JarEntry entry, byte[] buf) throws IOException, SecurityException {
 
         if (DEBUG) {
