@@ -61,22 +61,22 @@ public class CMethodBindingEmitter extends FunctionEmitter {
   protected MethodBinding binding;
 
   /** Name of the package in which the corresponding Java method resides.*/
-  private String packageName;
+  private final String packageName;
 
   /** Name of the class in which the corresponding Java method resides.*/
-  private String className;
+  private final String className;
 
   /**
    * Whether or not the Java<->C JNI binding for this emitter's MethodBinding
    * is overloaded.
    */
-  private boolean isOverloadedBinding;
+  private final boolean isOverloadedBinding;
 
   /**
    * Whether or not the Java-side of the Java<->C JNI binding for this
    * emitter's MethodBinding is static.
    */
-  private boolean isJavaMethodStatic;
+  private final boolean isJavaMethodStatic;
 
   // Flags which change various aspects of glue code generation
   protected boolean forImplementingMethodCall;
@@ -427,7 +427,9 @@ public class CMethodBindingEmitter extends FunctionEmitter {
           writer.println("  jsize _tmpArrayLen;");
 
           // Pointer to the data in the Buffer, taking the offset into account
-          writer.println("  int * _offsetHandle = NULL;");
+          if(type.isNIOBufferArray()) {
+            writer.println("  int * _offsetHandle = NULL;");
+          }
 
           emittedDataCopyTemps = true;
         }
@@ -925,6 +927,15 @@ public class CMethodBindingEmitter extends FunctionEmitter {
         if (isConstPtrPtr(cArgType)) {
             writer.print("const ");
         }
+
+        // if this is a pointer to an unsigned type, add unsigned to the name to avoid compiler warnings
+        if(cArgType.isPointer()) {
+          Type typeLast = ((PointerType)cArgType).getLastTargetType();
+          if(typeLast.isInt() && (((IntType)typeLast).isPrimitiveUnsigned())) {
+            writer.print("unsigned ");
+          }
+        }
+
         writer.print(cArgType.getName());
         writer.print(") ");
         if (binding.getCArgumentType(i).isPointer() && javaArgType.isPrimitive()) {
@@ -1048,11 +1059,11 @@ public class CMethodBindingEmitter extends FunctionEmitter {
         writer.println("  " + arrayRes + " = (*env)->NewObjectArray(env, " + arrayResLength + ", (*env)->FindClass(env, \"java/nio/ByteBuffer\"), NULL);");
         writer.println("  for (" + arrayIdx + " = 0; " + arrayIdx + " < " + arrayResLength + "; " + arrayIdx + "++) {");
         Type retType = binding.getCSymbol().getReturnType();
-	Type pointerType;
+        Type pointerType;
         if (retType.isPointer()) {
-	  pointerType = retType.asPointer().getTargetType();
+          pointerType = retType.asPointer().getTargetType();
         } else {
-	  pointerType = retType.asArray().getElementType();
+          pointerType = retType.asArray().getElementType();
         }
         writer.println("    (*env)->SetObjectArrayElement(env, " + arrayRes + ", " + arrayIdx +
                        ", (*env)->NewDirectByteBuffer(env, _res[" + arrayIdx + "], sizeof(" + pointerType.getName() + ")));");
