@@ -12,6 +12,10 @@ import com.jogamp.common.os.MachineDescription;
 import com.jogamp.common.os.Platform;
 import com.jogamp.junit.util.JunitTracer;
 
+import org.junit.FixMethodOrder;
+import org.junit.runners.MethodSorters;
+
+@FixMethodOrder(MethodSorters.NAME_ASCENDING)
 public class TestStructAccessorEndian extends JunitTracer {
 
     @Test
